package com.qmth.boot.core.rateLimit.service.impl;

import com.qmth.boot.core.rateLimit.entity.RateLimitRule;
import com.qmth.boot.core.rateLimit.entity.RateLimitTarget;
import com.qmth.boot.core.rateLimit.service.RateLimitService;
import org.apache.commons.lang3.ArrayUtils;

import java.time.Duration;
import java.util.HashMap;
import java.util.Map;

public class MemoryRateLimitService implements RateLimitService {

    protected static final String KEY_JOINER = "_";

    protected static final int LIMITOR_ARRAY_COUNT = 10;

    private final Map<String, RateLimitor[]> limitorMap = new HashMap<>();

    @Override
    public boolean accept(String endpoint, String device, RateLimitRule... rules) {
        if (ArrayUtils.isEmpty(rules)) {
            return true;
        }
        for (RateLimitRule rule : rules) {
            if (!accept(endpoint, device, rule)) {
                return false;
            }
        }
        return true;
    }

    protected boolean accept(String endpoint, String device, RateLimitRule rule) {
        String key = endpoint;
        if (rule.getTarget() == RateLimitTarget.DEVICE) {
            // 若限流对象是单个设备，则KEY需要包含设备标识
            key = key.concat(KEY_JOINER).concat(device);
        }
        RateLimitor limitor = getLimitor(key, rule.getPeriod());
        return limitor.count() <= rule.getCount() && limitor.incr() <= rule.getCount();
    }

    private RateLimitor getLimitor(String key, Duration period) {
        RateLimitor[] array = limitorMap.get(key);
        if (array == null) {
            synchronized (limitorMap) {
                array = limitorMap.computeIfAbsent(key, k -> initArray());
            }
        }
        long time = System.currentTimeMillis() / period.toMillis();
        int index = (int) (time % LIMITOR_ARRAY_COUNT);
        RateLimitor limitor = array[index];
        // 判断当前限流器归属的时间窗口，不是当前则重置
        if (limitor.time() != time) {
            limitor.reset(time);
        }
        return limitor;
    }

    private RateLimitor[] initArray() {
        RateLimitor[] array = new RateLimitor[LIMITOR_ARRAY_COUNT];
        for (int i = 0; i < LIMITOR_ARRAY_COUNT; i++) {
            array[i] = new RateLimitor(0);
        }
        return array;
    }

}
